# bash completion for glustercli                           -*- shell-script -*-

__glustercli_debug()
{
    if [[ -n ${BASH_COMP_DEBUG_FILE} ]]; then
        echo "$*" >> "${BASH_COMP_DEBUG_FILE}"
    fi
}

# Homebrew on Macs have version 1.3 of bash-completion which doesn't include
# _init_completion. This is a very minimal version of that function.
__glustercli_init_completion()
{
    COMPREPLY=()
    _get_comp_words_by_ref "$@" cur prev words cword
}

__glustercli_index_of_word()
{
    local w word=$1
    shift
    index=0
    for w in "$@"; do
        [[ $w = "$word" ]] && return
        index=$((index+1))
    done
    index=-1
}

__glustercli_contains_word()
{
    local w word=$1; shift
    for w in "$@"; do
        [[ $w = "$word" ]] && return
    done
    return 1
}

__glustercli_handle_reply()
{
    __glustercli_debug "${FUNCNAME[0]}"
    case $cur in
        -*)
            if [[ $(type -t compopt) = "builtin" ]]; then
                compopt -o nospace
            fi
            local allflags
            if [ ${#must_have_one_flag[@]} -ne 0 ]; then
                allflags=("${must_have_one_flag[@]}")
            else
                allflags=("${flags[*]} ${two_word_flags[*]}")
            fi
            COMPREPLY=( $(compgen -W "${allflags[*]}" -- "$cur") )
            if [[ $(type -t compopt) = "builtin" ]]; then
                [[ "${COMPREPLY[0]}" == *= ]] || compopt +o nospace
            fi

            # complete after --flag=abc
            if [[ $cur == *=* ]]; then
                if [[ $(type -t compopt) = "builtin" ]]; then
                    compopt +o nospace
                fi

                local index flag
                flag="${cur%=*}"
                __glustercli_index_of_word "${flag}" "${flags_with_completion[@]}"
                COMPREPLY=()
                if [[ ${index} -ge 0 ]]; then
                    PREFIX=""
                    cur="${cur#*=}"
                    ${flags_completion[${index}]}
                    if [ -n "${ZSH_VERSION}" ]; then
                        # zsh completion needs --flag= prefix
                        eval "COMPREPLY=( \"\${COMPREPLY[@]/#/${flag}=}\" )"
                    fi
                fi
            fi
            return 0;
            ;;
    esac

    # check if we are handling a flag with special work handling
    local index
    __glustercli_index_of_word "${prev}" "${flags_with_completion[@]}"
    if [[ ${index} -ge 0 ]]; then
        ${flags_completion[${index}]}
        return
    fi

    # we are parsing a flag and don't have a special handler, no completion
    if [[ ${cur} != "${words[cword]}" ]]; then
        return
    fi

    local completions
    completions=("${commands[@]}")
    if [[ ${#must_have_one_noun[@]} -ne 0 ]]; then
        completions=("${must_have_one_noun[@]}")
    fi
    if [[ ${#must_have_one_flag[@]} -ne 0 ]]; then
        completions+=("${must_have_one_flag[@]}")
    fi
    COMPREPLY=( $(compgen -W "${completions[*]}" -- "$cur") )

    if [[ ${#COMPREPLY[@]} -eq 0 && ${#noun_aliases[@]} -gt 0 && ${#must_have_one_noun[@]} -ne 0 ]]; then
        COMPREPLY=( $(compgen -W "${noun_aliases[*]}" -- "$cur") )
    fi

    if [[ ${#COMPREPLY[@]} -eq 0 ]]; then
		if declare -F __glustercli_custom_func >/dev/null; then
			# try command name qualified custom func
			__glustercli_custom_func
		else
			# otherwise fall back to unqualified for compatibility
			declare -F ___custom_func >/dev/null && __custom_func
		fi
    fi

    # available in bash-completion >= 2, not always present on macOS
    if declare -F __ltrim_colon_completions >/dev/null; then
        __ltrim_colon_completions "$cur"
    fi

    # If there is only 1 completion and it is a flag with an = it will be completed
    # but we don't want a space after the =
    if [[ "${#COMPREPLY[@]}" -eq "1" ]] && [[ $(type -t compopt) = "builtin" ]] && [[ "${COMPREPLY[0]}" == --*= ]]; then
       compopt -o nospace
    fi
}

# The arguments should be in the form "ext1|ext2|extn"
__glustercli_handle_filename_extension_flag()
{
    local ext="$1"
    _filedir "@(${ext})"
}

__glustercli_handle_subdirs_in_dir_flag()
{
    local dir="$1"
    pushd "${dir}" >/dev/null 2>&1 && _filedir -d && popd >/dev/null 2>&1
}

__glustercli_handle_flag()
{
    __glustercli_debug "${FUNCNAME[0]}: c is $c words[c] is ${words[c]}"

    # if a command required a flag, and we found it, unset must_have_one_flag()
    local flagname=${words[c]}
    local flagvalue
    # if the word contained an =
    if [[ ${words[c]} == *"="* ]]; then
        flagvalue=${flagname#*=} # take in as flagvalue after the =
        flagname=${flagname%=*} # strip everything after the =
        flagname="${flagname}=" # but put the = back
    fi
    __glustercli_debug "${FUNCNAME[0]}: looking for ${flagname}"
    if __glustercli_contains_word "${flagname}" "${must_have_one_flag[@]}"; then
        must_have_one_flag=()
    fi

    # if you set a flag which only applies to this command, don't show subcommands
    if __glustercli_contains_word "${flagname}" "${local_nonpersistent_flags[@]}"; then
      commands=()
    fi

    # keep flag value with flagname as flaghash
    # flaghash variable is an associative array which is only supported in bash > 3.
    if [[ -z "${BASH_VERSION}" || "${BASH_VERSINFO[0]}" -gt 3 ]]; then
        if [ -n "${flagvalue}" ] ; then
            flaghash[${flagname}]=${flagvalue}
        elif [ -n "${words[ $((c+1)) ]}" ] ; then
            flaghash[${flagname}]=${words[ $((c+1)) ]}
        else
            flaghash[${flagname}]="true" # pad "true" for bool flag
        fi
    fi

    # skip the argument to a two word flag
    if __glustercli_contains_word "${words[c]}" "${two_word_flags[@]}"; then
        c=$((c+1))
        # if we are looking for a flags value, don't show commands
        if [[ $c -eq $cword ]]; then
            commands=()
        fi
    fi

    c=$((c+1))

}

__glustercli_handle_noun()
{
    __glustercli_debug "${FUNCNAME[0]}: c is $c words[c] is ${words[c]}"

    if __glustercli_contains_word "${words[c]}" "${must_have_one_noun[@]}"; then
        must_have_one_noun=()
    elif __glustercli_contains_word "${words[c]}" "${noun_aliases[@]}"; then
        must_have_one_noun=()
    fi

    nouns+=("${words[c]}")
    c=$((c+1))
}

__glustercli_handle_command()
{
    __glustercli_debug "${FUNCNAME[0]}: c is $c words[c] is ${words[c]}"

    local next_command
    if [[ -n ${last_command} ]]; then
        next_command="_${last_command}_${words[c]//:/__}"
    else
        if [[ $c -eq 0 ]]; then
            next_command="_glustercli_root_command"
        else
            next_command="_${words[c]//:/__}"
        fi
    fi
    c=$((c+1))
    __glustercli_debug "${FUNCNAME[0]}: looking for ${next_command}"
    declare -F "$next_command" >/dev/null && $next_command
}

__glustercli_handle_word()
{
    if [[ $c -ge $cword ]]; then
        __glustercli_handle_reply
        return
    fi
    __glustercli_debug "${FUNCNAME[0]}: c is $c words[c] is ${words[c]}"
    if [[ "${words[c]}" == -* ]]; then
        __glustercli_handle_flag
    elif __glustercli_contains_word "${words[c]}" "${commands[@]}"; then
        __glustercli_handle_command
    elif [[ $c -eq 0 ]]; then
        __glustercli_handle_command
    elif __glustercli_contains_word "${words[c]}" "${command_aliases[@]}"; then
        # aliashash variable is an associative array which is only supported in bash > 3.
        if [[ -z "${BASH_VERSION}" || "${BASH_VERSINFO[0]}" -gt 3 ]]; then
            words[c]=${aliashash[${words[c]}]}
            __glustercli_handle_command
        else
            __glustercli_handle_noun
        fi
    else
        __glustercli_handle_noun
    fi
    __glustercli_handle_word
}

_glustercli_bitrot_disable()
{
    last_command="glustercli_bitrot_disable"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_bitrot_enable()
{
    last_command="glustercli_bitrot_enable"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_bitrot_scrub()
{
    last_command="glustercli_bitrot_scrub"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_bitrot_scrub-freq()
{
    last_command="glustercli_bitrot_scrub-freq"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_bitrot_scrub-throttle()
{
    last_command="glustercli_bitrot_scrub-throttle"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_bitrot()
{
    last_command="glustercli_bitrot"

    command_aliases=()

    commands=()
    commands+=("disable")
    commands+=("enable")
    commands+=("scrub")
    commands+=("scrub-freq")
    commands+=("scrub-throttle")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_device_add()
{
    last_command="glustercli_device_add"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_device()
{
    last_command="glustercli_device"

    command_aliases=()

    commands=()
    commands+=("add")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_events_webhook-add()
{
    last_command="glustercli_events_webhook-add"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--bearer-token=")
    two_word_flags+=("-t")
    local_nonpersistent_flags+=("--bearer-token=")
    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_events_webhook-del()
{
    last_command="glustercli_events_webhook-del"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_events_webhooks()
{
    last_command="glustercli_events_webhooks"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_events()
{
    last_command="glustercli_events"

    command_aliases=()

    commands=()
    commands+=("webhook-add")
    commands+=("webhook-del")
    commands+=("webhooks")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_geo-replication_create()
{
    last_command="glustercli_geo-replication_create"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--force")
    flags+=("-f")
    local_nonpersistent_flags+=("--force")
    flags+=("--remote-endpoints=")
    local_nonpersistent_flags+=("--remote-endpoints=")
    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_geo-replication_delete()
{
    last_command="glustercli_geo-replication_delete"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--force")
    flags+=("-f")
    local_nonpersistent_flags+=("--force")
    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_geo-replication_get()
{
    last_command="glustercli_geo-replication_get"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--show-all")
    flags+=("-a")
    local_nonpersistent_flags+=("--show-all")
    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_geo-replication_pause()
{
    last_command="glustercli_geo-replication_pause"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--force")
    flags+=("-f")
    local_nonpersistent_flags+=("--force")
    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_geo-replication_reset()
{
    last_command="glustercli_geo-replication_reset"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_geo-replication_resume()
{
    last_command="glustercli_geo-replication_resume"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--force")
    flags+=("-f")
    local_nonpersistent_flags+=("--force")
    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_geo-replication_set()
{
    last_command="glustercli_geo-replication_set"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_geo-replication_start()
{
    last_command="glustercli_geo-replication_start"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--force")
    flags+=("-f")
    local_nonpersistent_flags+=("--force")
    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_geo-replication_status()
{
    last_command="glustercli_geo-replication_status"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_geo-replication_stop()
{
    last_command="glustercli_geo-replication_stop"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--force")
    flags+=("-f")
    local_nonpersistent_flags+=("--force")
    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_geo-replication()
{
    last_command="glustercli_geo-replication"

    command_aliases=()

    commands=()
    commands+=("create")
    commands+=("delete")
    commands+=("get")
    commands+=("pause")
    commands+=("reset")
    commands+=("resume")
    commands+=("set")
    commands+=("start")
    commands+=("status")
    commands+=("stop")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_peer_add()
{
    last_command="glustercli_peer_add"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_peer_list()
{
    last_command="glustercli_peer_list"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--key=")
    local_nonpersistent_flags+=("--key=")
    flags+=("--value=")
    local_nonpersistent_flags+=("--value=")
    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_peer_remove()
{
    last_command="glustercli_peer_remove"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--force")
    flags+=("-f")
    local_nonpersistent_flags+=("--force")
    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_peer_status()
{
    last_command="glustercli_peer_status"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_peer()
{
    last_command="glustercli_peer"

    command_aliases=()

    commands=()
    commands+=("add")
    commands+=("list")
    commands+=("remove")
    commands+=("status")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_snapshot_activate()
{
    last_command="glustercli_snapshot_activate"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--force")
    flags+=("-f")
    local_nonpersistent_flags+=("--force")
    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_snapshot_clone()
{
    last_command="glustercli_snapshot_clone"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_snapshot_create()
{
    last_command="glustercli_snapshot_create"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--desctription=")
    local_nonpersistent_flags+=("--desctription=")
    flags+=("--force")
    local_nonpersistent_flags+=("--force")
    flags+=("--timestamp")
    local_nonpersistent_flags+=("--timestamp")
    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_snapshot_deactivate()
{
    last_command="glustercli_snapshot_deactivate"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_snapshot_delete_all()
{
    last_command="glustercli_snapshot_delete_all"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_snapshot_delete()
{
    last_command="glustercli_snapshot_delete"

    command_aliases=()

    commands=()
    commands+=("all")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_snapshot_info()
{
    last_command="glustercli_snapshot_info"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_snapshot_list()
{
    last_command="glustercli_snapshot_list"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_snapshot_restore()
{
    last_command="glustercli_snapshot_restore"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_snapshot_status()
{
    last_command="glustercli_snapshot_status"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_snapshot()
{
    last_command="glustercli_snapshot"

    command_aliases=()

    commands=()
    commands+=("activate")
    commands+=("clone")
    commands+=("create")
    commands+=("deactivate")
    commands+=("delete")
    commands+=("info")
    commands+=("list")
    commands+=("restore")
    commands+=("status")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_volume_add-brick()
{
    last_command="glustercli_volume_add-brick"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--allow-mount-as-brick")
    local_nonpersistent_flags+=("--allow-mount-as-brick")
    flags+=("--allow-root-dir")
    local_nonpersistent_flags+=("--allow-root-dir")
    flags+=("--create-brick-dir")
    local_nonpersistent_flags+=("--create-brick-dir")
    flags+=("--force")
    flags+=("-f")
    local_nonpersistent_flags+=("--force")
    flags+=("--replica=")
    local_nonpersistent_flags+=("--replica=")
    flags+=("--reuse-bricks")
    local_nonpersistent_flags+=("--reuse-bricks")
    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_volume_create()
{
    last_command="glustercli_volume_create"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--allow-advanced-options")
    local_nonpersistent_flags+=("--allow-advanced-options")
    flags+=("--allow-deprecated-options")
    local_nonpersistent_flags+=("--allow-deprecated-options")
    flags+=("--allow-experimental-options")
    local_nonpersistent_flags+=("--allow-experimental-options")
    flags+=("--allow-mount-as-brick")
    local_nonpersistent_flags+=("--allow-mount-as-brick")
    flags+=("--allow-root-dir")
    local_nonpersistent_flags+=("--allow-root-dir")
    flags+=("--arbiter=")
    local_nonpersistent_flags+=("--arbiter=")
    flags+=("--create-brick-dir")
    local_nonpersistent_flags+=("--create-brick-dir")
    flags+=("--disperse=")
    local_nonpersistent_flags+=("--disperse=")
    flags+=("--disperse-data=")
    local_nonpersistent_flags+=("--disperse-data=")
    flags+=("--distribute=")
    local_nonpersistent_flags+=("--distribute=")
    flags+=("--enable-snapshot")
    local_nonpersistent_flags+=("--enable-snapshot")
    flags+=("--exclude-peers=")
    local_nonpersistent_flags+=("--exclude-peers=")
    flags+=("--exclude-zones=")
    local_nonpersistent_flags+=("--exclude-zones=")
    flags+=("--force")
    local_nonpersistent_flags+=("--force")
    flags+=("--limit-peers=")
    local_nonpersistent_flags+=("--limit-peers=")
    flags+=("--limit-zones=")
    local_nonpersistent_flags+=("--limit-zones=")
    flags+=("--options=")
    local_nonpersistent_flags+=("--options=")
    flags+=("--redundancy=")
    local_nonpersistent_flags+=("--redundancy=")
    flags+=("--replica=")
    local_nonpersistent_flags+=("--replica=")
    flags+=("--reuse-bricks")
    local_nonpersistent_flags+=("--reuse-bricks")
    flags+=("--size=")
    local_nonpersistent_flags+=("--size=")
    flags+=("--snapshot-reserve-factor=")
    local_nonpersistent_flags+=("--snapshot-reserve-factor=")
    flags+=("--stripe=")
    local_nonpersistent_flags+=("--stripe=")
    flags+=("--subvols-zones-overlap")
    local_nonpersistent_flags+=("--subvols-zones-overlap")
    flags+=("--thin-arbiter=")
    local_nonpersistent_flags+=("--thin-arbiter=")
    flags+=("--transport=")
    local_nonpersistent_flags+=("--transport=")
    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_volume_delete()
{
    last_command="glustercli_volume_delete"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_volume_edit-metadata()
{
    last_command="glustercli_volume_edit-metadata"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--delete")
    local_nonpersistent_flags+=("--delete")
    flags+=("--key=")
    local_nonpersistent_flags+=("--key=")
    flags+=("--value=")
    local_nonpersistent_flags+=("--value=")
    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_flag+=("--key=")
    must_have_one_flag+=("--value=")
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_volume_get()
{
    last_command="glustercli_volume_get"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--advanced")
    local_nonpersistent_flags+=("--advanced")
    flags+=("--basic")
    local_nonpersistent_flags+=("--basic")
    flags+=("--modified")
    local_nonpersistent_flags+=("--modified")
    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_volume_heal_full()
{
    last_command="glustercli_volume_heal_full"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_volume_heal_index()
{
    last_command="glustercli_volume_heal_index"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_volume_heal_info()
{
    last_command="glustercli_volume_heal_info"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--info-summary")
    local_nonpersistent_flags+=("--info-summary")
    flags+=("--split-brain-info")
    local_nonpersistent_flags+=("--split-brain-info")
    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_volume_heal()
{
    last_command="glustercli_volume_heal"

    command_aliases=()

    commands=()
    commands+=("full")
    commands+=("index")
    commands+=("info")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_volume_info()
{
    last_command="glustercli_volume_info"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--key=")
    local_nonpersistent_flags+=("--key=")
    flags+=("--value=")
    local_nonpersistent_flags+=("--value=")
    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_volume_list()
{
    last_command="glustercli_volume_list"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--key=")
    local_nonpersistent_flags+=("--key=")
    flags+=("--value=")
    local_nonpersistent_flags+=("--value=")
    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_volume_reset()
{
    last_command="glustercli_volume_reset"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--all")
    local_nonpersistent_flags+=("--all")
    flags+=("--force")
    local_nonpersistent_flags+=("--force")
    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_volume_set()
{
    last_command="glustercli_volume_set"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--advanced")
    local_nonpersistent_flags+=("--advanced")
    flags+=("--deprecated")
    local_nonpersistent_flags+=("--deprecated")
    flags+=("--experimental")
    local_nonpersistent_flags+=("--experimental")
    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_volume_size()
{
    last_command="glustercli_volume_size"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_volume_start()
{
    last_command="glustercli_volume_start"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--force")
    flags+=("-f")
    local_nonpersistent_flags+=("--force")
    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_volume_statedump()
{
    last_command="glustercli_volume_statedump"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--client=")
    local_nonpersistent_flags+=("--client=")
    flags+=("--quota")
    local_nonpersistent_flags+=("--quota")
    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_volume_status()
{
    last_command="glustercli_volume_status"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_volume_stop()
{
    last_command="glustercli_volume_stop"

    command_aliases=()

    commands=()

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--force")
    flags+=("-f")
    local_nonpersistent_flags+=("--force")
    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_volume()
{
    last_command="glustercli_volume"

    command_aliases=()

    commands=()
    commands+=("add-brick")
    commands+=("create")
    commands+=("delete")
    commands+=("edit-metadata")
    commands+=("get")
    commands+=("heal")
    commands+=("info")
    commands+=("list")
    commands+=("reset")
    commands+=("set")
    commands+=("size")
    commands+=("start")
    commands+=("statedump")
    commands+=("status")
    commands+=("stop")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

_glustercli_root_command()
{
    last_command="glustercli"

    command_aliases=()

    commands=()
    commands+=("bitrot")
    commands+=("device")
    commands+=("events")
    commands+=("geo-replication")
    commands+=("peer")
    commands+=("snapshot")
    commands+=("volume")

    flags=()
    two_word_flags=()
    local_nonpersistent_flags=()
    flags_with_completion=()
    flags_completion=()

    flags+=("--cacert=")
    flags+=("--endpoints=")
    flags+=("--insecure")
    flags+=("--json")
    flags+=("--loglevel=")
    flags+=("--script-mode")
    flags+=("-y")
    flags+=("--secret=")
    flags+=("--secret-file=")
    flags+=("--timeout=")
    flags+=("--user=")
    flags+=("--verbose")
    flags+=("-v")
    flags+=("--xml")

    must_have_one_flag=()
    must_have_one_noun=()
    noun_aliases=()
}

__start_glustercli()
{
    local cur prev words cword
    declare -A flaghash 2>/dev/null || :
    declare -A aliashash 2>/dev/null || :
    if declare -F _init_completion >/dev/null 2>&1; then
        _init_completion -s || return
    else
        __glustercli_init_completion -n "=" || return
    fi

    local c=0
    local flags=()
    local two_word_flags=()
    local local_nonpersistent_flags=()
    local flags_with_completion=()
    local flags_completion=()
    local commands=("glustercli")
    local must_have_one_flag=()
    local must_have_one_noun=()
    local last_command
    local nouns=()

    __glustercli_handle_word
}

if [[ $(type -t compopt) = "builtin" ]]; then
    complete -o default -F __start_glustercli glustercli
else
    complete -o default -o nospace -F __start_glustercli glustercli
fi

# ex: ts=4 sw=4 et filetype=sh
